from kiwi.ui.dialogs import info

from gazpacho.plugins import Plugin

import dbus
import dbus.service
import dbus.glib
import gobject
from pprint import pprint

class Server(dbus.service.Object):
    def __init__(self, bus_name, plugin, object_path="/Gazpacho"):
	self.plugin = plugin
        dbus.service.Object.__init__(self, bus_name, object_path)

    @dbus.service.method("biz.sicem.gazpacho.GazpachoInterface")
    def GetProjectList(self):
        return self.plugin.projects.keys()

    @dbus.service.method("biz.sicem.gazpacho.GazpachoInterface")
    def GetWidgetList(self, projectName):
        ret = []
        if projectName in self.plugin.projects:
            signal, project = self.plugin.projects[projectName]
            ret = [w.get_name() for w in project.get_widgets()]
        return ret

    @dbus.service.signal("biz.sicem.gazpacho.GazpachoService")
    def AddSignal(self, projectName, widgetName, widgetType, signalName, signalHandlerName):
        pass

class ProjectInfo:
    def __init__(self, project):
        self.project = project
        self.signals = []
        self.gadget_signals = {}

class PepinoPlugin(Plugin):

    def activate(self, app):
	self.projects = {}
        self.window = app.get_window()

	session_bus = dbus.SessionBus()
	name = dbus.service.BusName("biz.sicem.gazpacho.GazpachoService", bus=session_bus)
	self.server = Server(name, self)

	app.connect("set-project", self.on_app__set_project)

    def on_app__set_project(self, app, project):
	if project.name not in self.projects:
            pi = self.projects[project.name] = ProjectInfo(project)
            pi.signals.append(project.connect("changed",
                                              self.on_project__changed))
            pi.signals.append(project.connect("add-gadget",
                                              self.on_project__add_gadget))
            pi.signals.append(project.connect("remove-gadget",
                                              self.on_project__remove_gadget))

    def on_project__changed(self, project):
	pass

    def on_project__add_gadget(self, project, gadget):
        if project.name in self.projects:
            pi = self.projects[project.name]
            add_id = gadget.connect("add-signal",
                                    self.on_gadget__add_signal)
            rem_id = gadget.connect("remove-signal",
                                    self.on_gadget__remove_signal)
            pi.gadget_signals[gadget] = [add_id, rem_id]

    def on_project__remove_gadget(self, project, gadget):
        if project.name in self.projects:
            pi = self.projects[project.name]
            for signal_id in pi.gadget_signals[gadget]:
                gadget.disconnect(signal_id)
            del pi.gadget_signals[gadget]

    def on_gadget__add_signal(self, gadget, signal_info):
        pprint(signal_info)
        self.server.AddSignal(gadget.project.name,
			      gadget.name,
                              gobject.type_name(gadget.widget),
                              signal_info.normalized_name,
                              signal_info.handler)

    def on_gadget__remove_signal(self, gadget, signal_info):
        pprint(signal_info)

    def deactivate(self):
        for project_info in self.projects.values():
            for signal_id in project_info.signals:
                project_info.project.disconnect(signal_id)

            for gadget, signals in project_info.gadget_signals.items():
                for signal in signals:
                    gadget.disconnect(signal)
