import gedit, gtk
import dbus
import dbus.glib
from xml.dom.minidom import parse
import os

ui_str = """
<ui>
  <menubar name="MenuBar">
    <menu name="EditMenu" action="Edit">
      <placeholder name="EditOps_4">
        <menuitem name="Pepino" action="Pepino"/>
      </placeholder>
    </menu>
  </menubar>
</ui>
"""

class PepinoPlugin(gedit.Plugin):
	# Required
	def __init__(self):
		gedit.Plugin.__init__(self)

	# Required
	def activate(self, window):
		manager = window.get_ui_manager()
		self.data = dict()
		self.window = window

		self.data["action_group"] = gtk.ActionGroup("GeditPepinoPluginActions")
		self.data["action_group"].add_actions(
		         [("Pepino", None, "Pepino for Gazpacho", None, 
		           "A Pepino conector for Gazpacho.  Taste it!",
		           lambda a, w: self.pepino(w))],
		         window)
		manager.insert_action_group(self.data["action_group"], -1)
		self.data["ui_id"] = manager.add_ui_from_string(ui_str)

		print "Pepino activated"

		# DBUS Interface initialization
		bus = dbus.SessionBus()
		self.object = bus.get_object("biz.sicem.gazpacho.GazpachoService",
		                             "/Gazpacho")

		# Panel
		side_panel = window.get_side_panel()
		self.panel = PepinoPanel(bus)
		self.panel.show_all()
		side_panel.add_item(self.panel, "Pepino", gtk.STOCK_CONVERT)

		self.object.connect_to_signal("AddSignal",
                         self.pepino_signal_handler,
                         dbus_interface="biz.sicem.gazpacho.GazpachoService")

		dom = parse(os.environ['HOME'] + ".gnome2/gedit/gtk-api.xml")
		data = dom.getElementsByTagName("signal")
		self.signals = {}

		for signal in dom.getElementsByTagName("signal"):
        		name = signal.getAttribute("cname").replace("_", "-")
        		parent = signal.parentNode.getAttribute("cname")
        		parameters = []
        		for parameter in signal.getElementsByTagName("parameter"):
                		parameters.append(parameter.getAttribute("name"))

        		self.signals["%s::%s" % (parent,name)] = parameters

		self.update_ui(window)

	# Required
	def deactivate(self, window):
		manager = window.get_ui_manager()
		manager.remove_ui(self.data["ui_id"])
		manager.remove_action_group(self.data["action_group"])
		
		side_panel = window.get_side_panel()
		side_panel.remove_item(self.panel)

		manager.ensure_update()

	# Required
	def update_ui(self, window):
		view = window.get_active_view()
		self.data["action_group"].set_sensitive(view is not None and \
		                                        view.get_editable())

	def pepino_signal_handler(self, project_name, widget_name, widget_type, 
	                          signal_name, signal_handler_name):
		print "Callback llamado"
    		print project_name, widget_name, signal_name, signal_handler_name
    		print self.signals["%s::%s" % (widget_type, signal_name)]

		if project_name == self.panel.get_project():
			document = self.window.get_active_document()
			parameters = self.signals["%s::%s" % (widget_type, signal_name)]
			code = "def %s(%s):\n\t" % (signal_handler_name, 
			                            ",".join(parameters))
			document.begin_user_action()
			document.insert_at_cursor(code)
			document.end_user_action()

			self.panel.update_widget_list()

class PepinoPanel(gtk.VBox):
	def __init__(self, bus):
		gtk.VBox.__init__(self)

		# List of projects
		self.projects = gtk.ComboBox()
		model = gtk.ListStore(str)

		renderer = gtk.CellRendererText()
		self.projects.pack_start(renderer)
		self.projects.add_attribute(renderer, 'text', 0)

		# get data from bus
		remote_object = bus.get_object("biz.sicem.gazpacho.GazpachoService",
		                               "/Gazpacho")
		self.iface = dbus.Interface(remote_object, 
		                            "biz.sicem.gazpacho.GazpachoInterface")

		project_list = self.iface.GetProjectList()
		for project in project_list:
			print project
			model.append([project])

		self.projects.set_model(model)
		self.projects.set_active(0)

		self.pack_start(self.projects, expand=False, fill=False)

		# List of widgets and signals
		self.widgets = gtk.TreeView()
		model = gtk.ListStore(str)
		self.widgets.set_model(model)		
		self.widgets.set_headers_visible(False)

		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn(None, renderer, text=1)
		self.widgets.append_column(column)

		self.pack_start(self.widgets)

		self.projects.connect('changed', self.on_projects__changed)
		self.on_projects__changed(self.projects)

	def get_project(self):
		index = self.projects.get_active()
		print index
		model = self.projects.get_model()
		return model[index][0]

	def update_widget_list(self, project):
		print "project: ", project
		if project is None:
			project = self.get_project()

		widget_list = self.iface.GetWidgetList(project)
		print widget_list

	#widgets = self.iface.GetWidgetList()
	#for widget in projects:
	#	print widget
	#	model.append([widget])

	def on_projects__changed(self, projects, user_data=None):
		print "on_projects__changed"
		model = projects.get_model()
		print projects.get_active()

